/*
 *  nautilus-flac-converter.c
 * 
 *  Copyright (C) 2006 Brian Pepple
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Author: Brian Pepple <bdpepple@gmail.com>
 * 
 */

#ifdef HAVE_CONFIG_H
 #include <config.h> /* for GETTEXT_PACKAGE */
#endif

#include "nautilus-flac-converter.h"
#include "nautilus-flac-ogg.h"

#include <libnautilus-extension/nautilus-menu-provider.h>

#include <glib/gi18n-lib.h>
#include <gtk/gtkicontheme.h>
#include <gtk/gtkwidget.h>

#include <string.h> /* For strcmp */

static void nautilus_flac_converter_instance_init (NautilusFlacConverter      *flac);
static void nautilus_flac_converter_class_init    (NautilusFlacConverterClass *class);

static GType flac_converter_type = 0;

static gboolean
flac_converter_file_is_flac (NautilusFileInfo *file_info)
{
	gchar *uri_scheme;
	gchar *mime_type;
	gboolean maybe_flac;

	maybe_flac = TRUE;
	uri_scheme = nautilus_file_info_get_uri_scheme (file_info);

	if (strcmp (uri_scheme, "file") != 0)
		maybe_flac = FALSE;
	g_free (uri_scheme);

	mime_type = nautilus_file_info_get_mime_type (file_info);
	if (strncmp (mime_type, "audio/x-flac", 12) != 0)
		maybe_flac = FALSE;
	g_free (mime_type);

	return maybe_flac;
}

static GList *
flac_converter_filter_flacs (GList *files)
{
	GList *flacs;
	GList *file;

	flacs = NULL;

	for (file = files; file != NULL; file = file->next) {
		if (flac_converter_file_is_flac (file->data))
			flacs = g_list_prepend (flacs, file->data);
	}

	return flacs;
}

static void
flac_convert_callback (NautilusMenuItem *item,
		       GList            *files)
{
	NautilusFlacOgg *convert = nautilus_flac_ogg_new (flac_converter_filter_flacs (files));
	nautilus_flac_ogg_show_dialog (convert);
}

static GList *
nautilus_flac_converter_get_background_items (NautilusMenuProvider  *provider,
					      GtkWidget             *window,
					      NautilusFileInfo      *file_info)
{
	return NULL;
}

GList *
nautilus_flac_converter_get_file_items (NautilusMenuProvider *provider,
					GtkWidget            *window,
					GList                *files)
{
	NautilusMenuItem *item;
	GList *file;
	GList *items = NULL;

	for (file = files; file != NULL; file = file->next) {
		if (flac_converter_file_is_flac (file->data)) {
			item = nautilus_menu_item_new ("NautilusFlacConverter::convert",
						       _("_Convert FLAC to Ogg..."),
						       _("Convert each selected FLAC to an Ogg"),
						       "stock_insert-sound-plugin");

			g_signal_connect (item, "activate",
					  G_CALLBACK (flac_convert_callback),
					  nautilus_file_info_list_copy (files));

			items = g_list_prepend (items, item);
			items = g_list_reverse (items);

			return items;
		}
	}

	return NULL;
}

static void
nautilus_flac_converter_menu_provider_iface_init (NautilusMenuProviderIface *iface)
{
	iface->get_background_items = nautilus_flac_converter_get_background_items;
	iface->get_file_items = nautilus_flac_converter_get_file_items;
}

static void
nautilus_flac_converter_instance_init (NautilusFlacConverter *flac)
{
}

static void
nautilus_flac_converter_class_init (NautilusFlacConverterClass *class)
{
}

GType
nautilus_flac_converter_get_type (void)
{
	return flac_converter_type;
}

void
nautilus_flac_converter_register_type (GTypeModule *module)
{
	static const GTypeInfo info = {
		sizeof (NautilusFlacConverterClass),
		(GBaseInitFunc) NULL,
		(GBaseFinalizeFunc) NULL,
		(GClassInitFunc) nautilus_flac_converter_class_init,
		NULL,
		NULL,
		sizeof (NautilusFlacConverter),
		0,
		(GInstanceInitFunc) nautilus_flac_converter_instance_init,
	};

	static const GInterfaceInfo menu_provider_iface_info = {
		(GInterfaceInitFunc) nautilus_flac_converter_menu_provider_iface_init,
		NULL,
		NULL,
	};

	flac_converter_type = g_type_module_register_type (module,
							   G_TYPE_OBJECT,
							   "NautilusFlacConverter",
							   &info, 0);

	g_type_module_add_interface (module,
				     flac_converter_type,
				     NAUTILUS_TYPE_MENU_PROVIDER,
				     &menu_provider_iface_info);
}
